			/* multivariate probit (MVP) model */
			/* simulated data and real data examples from Chan & Jeliazkov (2009) */

new;
cls;
format /rd 5,4;
seed1 = 82415811;
u1 = rndus(1,1,seed1);
clear u1;
library optmum,pgraph;
tstart = date;


		/* INPUT PARAMETERS FOR THE MCMC ITERATIONS */

n0 = 1000;			/* number of MCMC draws to leave out */
mc = 10000;			/* the number of posterior MCMC draws */
capn = n0 + mc;
prntit = round(mc/100);		/* print every prntit-th iteration */

		/* ESTIMATION SETTINGS */

realdata = 1;			/* use REAL (=1) vs SIMULATED (=0) data */

nu = 10;		/* d.f. for M-H proposal */
p = 1.5;		/* ARMH importance constant */
lnp = ln(p);	/* to avoid recomputing this every time */
tauarmh = 1.5;

if realdata == 0;
	n = 1500;	/* total number of units */
	q = 4;		/* number of dimensions */
	k = 4;
	Ommean = eye(q);
	rh0_ = q + 10;
	{v0_,R0_} = paramwish(rh0_,Ommean);  		/* Ominv ~ Wish(v0_,R0_) */
	Ominv = rndw(v0_,R0_);		/* propose a random Omega matrix */
	Om = invpd(Ominv);
	Om = corrvc(Om);	/* because here Om is in correlation form */

	Om[1,q] = 0.5;		/* try various values of particular correlations */
	Om[q,1] = 0.5;

		/* HIGH CORRELATION CASE */
	Om = eye(q);
	r = 1;	/* update to form diagonal elements of Toeplitz matrix */
	i = 1;
	do while (i le (q-1)) and (r gt 0.25);
		r = r - 0.25;
		Om = diagrv1(Om,r*ones(q,1),-i);		/* OK to have a longer vector in the middle */
		Om = diagrv1(Om,r*ones(q,1),i);
	i = i + 1;
	endo;
		/* LOW CORRELATION CASE */
	Om = eye(q);
	r = 1;	/* update to form diagonal elements of Toeplitz matrix */
	i = 1;
	do while (i le (q-1)) and (r gt 0.25);
		r = r/2;
		Om = diagrv1(Om,r*ones(q,1),-i);		/* OK to have a longer vector in the middle */
		Om = diagrv1(Om,r*ones(q,1),i);
	i = i + 1;
	endo;

	Ominv = invpd(Om);
	Omtru = Om;
	beta = 0.3*ones(k,1);
	btru = beta;

	z = zeros(n,q);
	X = rndn(n*q,k);
	y = zeros(n,q);			/* initialize */
	Xb = zeros(n,q);		/* to store Xt*beta */
	yi = zeros(q,1);		/* initialize */
	for i (1,n,1);
		Xb[i,.] = (X[(i-1)*q+1:i*q,.]*beta)';
		zi = Xb[i,.]' + chol(Om)'rndn(q,1);
		for ii (1,q,1);
			yi[ii] = zi[ii] gt 0;	/* binary indicator */
		endfor;
		y[i,.] = yi';	/* store yi */
		z[i,.] = zi';	/* store for use in trials of continuous vs discrete analysis when realdata = 0 */
	endfor;
	?;
	print /flush "Omtru " Omtru;
	?;
else;		/* real data */
		/* LOAD PSID DATA ON 1545 WOMEN */
	n = 1545;	/* total number of women */
	q = 7;		/* number of periods */
	load y[] = y1545.dat;
	y = reshape(y,1545,q);
	load X[10815,9] = x1545.dat;	/* X contains INTRCPT AGE RACE YRSEDU CH2 CH5 CH13 CH17 HDINC  */
		/* CPI adjustment */
	CPI = 72.6|82.4|90.9|96.5|99.6|103.9|107.6;	/* CPI 1979-1985 */
	CPI = CPI ./ 113.6;	/* standardize by CPI in 1987 */
	CPI = ones(n,1) .*. CPI;
	X[.,9] = (X[.,9]./1000) ./ CPI;	/* adjust by the CPI (=100 in 1987, in thousands of dollars) */
	X[.,9] = ln(X[.,9]);	/* ln(INC) */
	X = X[.,1 2]~(X[.,2]^2)~X[.,3:6 9];		/* age is modeled by a quadratic polynomial */
	k = cols(X);
endif;

		/* PRIORS */

	/* linear parameters */
Bcap0_ = eye(k);	/* precision */
b0_ = zeros(k,1);	/* mean */
B0b0_ = Bcap0_*b0_;
priorbetams = normalms(b0_,Bcap0_);

	/* elements of L OmgenCJ08ARMHoptmum uses this */
	/*  */
nr = q*(q-1)/2;		/* number of correlations */
nOm = q*(q+1)/2;	/* number of unique elements in Om */
Acap0_ = 1*eye(nr);	/* precision */
a0_ = zeros(nr,1);	/* mean */
A0a0_ = Acap0_*a0_;
priorams = normalms(a0_,Acap0_);

		/* STARTING VALUES */

arJ = 0;		/* counter for AR draws in ARMH algorithm */
armhJ = 0;		/* counter for MH acceptance in ARMH algorithm */

beta = b0_;	/* from prior */
a = zeros(nr,1);	/* starting value */
Om = eye(q);
Ominv = invpd(Om);


MHacc = zeros(q,1);	/* to compute M-H acceptance rate */
XX = X'X;			/* need not be re-computed every time */
Xb = zeros(n,q);	/* n*q matrix of means */
if realdata == 1;
	z = y - 1;			/* latent data matrix */
	/*z = zeros(n,q)*/
endif;
it = 1;

		/* OPTIMIZATION AND INTEGRATION SETTINGS */

__output = 0;			/* do not print (= 0) or print (= 1) */
_opmiter = 15;			/* maximum number of iterations */

mu_opt = 0;
s2_opt = 1;
J_opt = 3;
y_opt = 0;
d0_opt = 0;
Dcap0_opt = 0;

		/* STORAGE */

betam = zeros(k,mc);
bhatm = zeros(k,mc);
Bm = zeros(k*(k+1)/2,mc);
vechBinvm = zeros(k*(k+1)/2,mc);
pdfbm = zeros(1,mc);
cm = zeros(nr,mc);			/* storage for correlations in Omega */
Omm = zeros(q*(q+1)/2,mc);	/* storage for Omega */
Ominvm = Omm;				/* storage for Ominv (Jensen's inequality applies) */
pdfOminvm = zeros(1,mc);	/* for pdf averaging */
vechbbm = zeros(q*(q+1)/2,mc);
Xb = zeros(n,q);			/* store means */


	/* to improve proc zgen */
ix0 = seqa(1,1,q);
ixm = zeros(q-1,q);
for i (1,q,1);
	ix = ix0;
	ix[i] = miss(0,0);
	ix = packr(ix);
	ixm[.,i] = ix;
endfor;


		/* MCMC GENERATION PROCS */

	/* proc to generate beta */
	/* Xi is not in SUR form -- same beta for every period */
	/* Xb is a global storage matrix */

proc(4) = bgen_same_b(z,X,B0b0_,Bcap0_,Om,Ominv);
local XOX,XOz,i,j,zi,Xi,XO,Binv,B,bhat,beta;

XOX = zeros(k,k);
XOz = zeros(k,1);
for i (1,n,1);
	zi = z[i,.]';
	Xi = X[(i-1)*q+1:i*q,.];
	XO = Xi'Ominv;
	XOX = XOX + XO*Xi;
	XOz = XOz + XO*zi;
endfor;

Binv = (Bcap0_ + XOX);
B = invpd(Binv);
bhat = B*(B0b0_ + XOz);
beta = rndmvn(bhat,B);		/* the draw of beta */

for i (1,n,1);
	Xb[i,.] = (X[(i-1)*q+1:i*q,.]*beta)';
endfor;

retp(beta,bhat,Binv,Xb);
endp;


	/* MACRO TO GENERATE OMEGA */
	/* uses optimization to produce ARMH proposal density */
	/* tt = it - n0; */

proc(2) = OmgenCJ08ARMHoptmum(z,Xb,n,q,a0,Om0,tt);
local U,L0,lam,lamold,ahat,diff,cntr,bg,en,i,L0inv,V,f,g,retcode,Vh,lncb,acc,a1,r1,r0,almh;

U = z - Xb;			/* n*q matrix of errors */
L0 = eye(q);		/* old lower triangular matrix with ones on main diagonal */
lam = ones(q,1);	/* initialize for old lambdas used in MH proposal */

lamold = lam;	/* lam was initialized at ones(q,1) */
ahat = a0;		/* start iterations at current a0 */
diff = 1;		/* to store maxc(abs(lam-lamold)) */
cntr = 1;
do until diff lt 0.00001;		/* by changing this, one can determine how far from a0 one wants to start */
	lam = ones(q,1);	/* re-initialize */
	bg = 1; en = 1;		/* fill L0 */
	for i (2,q,1);
		L0[i,1:i-1] = ahat[bg:en]';
	bg = en + 1;en = en + i;
	endfor;
	L0inv = inv(L0);	/* fill lam */
	for i (2,q,1);		/* create lhat */
		lam[i] = lam[i] - (L0inv[i,1:i-1]^2)*lam[1:i-1];
	endfor;
	bg = 1; en = 1;		/* create V and ahat to be used as starting values for optimization */
	for i (2,q,1);		/* create lhat */
		V = invpd(Acap0_[bg:en,bg:en] + (U[.,1:i-1]'U[.,1:i-1])/lam[i]);
		ahat[bg:en] = V*(A0a0_[bg:en] + (-U[.,1:i-1]'U[.,i])/lam[i]);	/* note the minus sign */
	bg = en + 1;en = en + i;
	endfor;
	diff = maxc(abs(lam-lamold));		/* reset for next iteration */
	lamold = lam;
	cntr = cntr + 1;
endo;

_opalgr = 5;			/* use Newton-Raphson (=5) or BFGS (=2) */
_ophsprc=&BHHHhess1;
{ ahat,f,g,retcode } = optmum(&postMVPa,ahat);	/* optmum minimizes */
Vh = invpd(_opfhess);	/* final Hessian used as covariance matrix for h(.) */
Vh = tauarmh*Vh;		/* additional tuning */
	/* update importance constant */
lncb = lnp - postMVPa(ahat) - pdfmvt(ahat,ahat,Vh,nu);	/* NOTE: taking negative of postMVPa because of optmum */

	/* A-R step */
acc = 0;
cntr = 1;
do while acc lt 1;
	a1 = rndmvt(nu,ahat,Vh);	/* A-R proposal */
	bg = 1; en = 1;		/* create L0 and L0inv */
	for i (2,q,1);
		L0[i,1:i-1] = a1[bg:en]';
	bg = en + 1; en = en + i;
	endfor;
	L0inv = inv(L0);	/* new Linv based on new L */
	lam = ones(q,1);	/* re-initialize */
	for i (2,q,1);		/* produce new lam */
		lam[i] = lam[i] - (L0inv[i,1:i-1]^2)*lam[1:i-1];
	endfor;

	if minc(lam) gt 0;	/* pass positive definiteness */
		r1 = -postMVPa(a1) - lncb - pdfmvt(a1,ahat,Vh,nu);	/* lik + priorb - lncb - h(b) */
		acc = ln(rndu(1,1)) le r1;
	else;
		print /flush "NON-PD MATRIX REJECTED IN AR STEP " (tt+n0);
	endif;	/* otherwise the proposal is automatically rejected */

		/* counters */
	if tt gt 0;
		arJ = arJ + 1;	/* AR counter */
	endif;
	cntr = cntr + 1;
	if cntr gt 100;
		print /flush "CAUTION: BREAKING OUT OF ARMH LOOP IN ITERATION " (tt+n0);
		break;
	endif;
endo;

	/* M-H step */
r0 = -postMVPa(a0) - lncb - pdfmvt(a0,ahat,Vh,nu);
if r0 gt 0;		/* b0 is NOT in D under current h(); o.w. alMH = 1 */
	almh = (r1 gt 0)*r1 - r0;	/* for both b0 in D and b0 not in D */
	acc = ln(rndu(1,1)) le almh;
	if tt gt 0;
		armhJ = armhJ + acc;	/* MH counter for ARMH algorithm */
	endif;
	if acc == 0;	/* if rejected; otherwise it stays the same */
		a1 = a0;
	endif;
else;
	if tt gt 0;
		armhJ = armhJ + 1;	/* MH counter for ARMH algorithm */
	endif;
endif;

	/* final Om */
	/* create L0 and L0inv */
bg = 1; en = 1;
for i (2,q,1);
	L0[i,1:i-1] = a1[bg:en]';
bg = en + 1; en = en + i;
endfor;
L0inv = inv(L0);	/* new Linv based on new L */
lam = ones(q,1);	/* re-initialize */
for i (2,q,1);		/* produce new lam */
	lam[i] = lam[i] - (L0inv[i,1:i-1]^2)*lam[1:i-1];
endfor;

Om = L0inv*diagrv(eye(q),lam)*L0inv';	/* the correlation matrix */

retp(a1,Om);
endp;


		/* posterior for optimization */
		/* globals: z, Xb, a0, Acap0_ */

proc postMVPa(a);
local L0,lam,bg,en,i,L0inv,Omlik,pdf;

L0 = eye(q);		/* lower unitriangular matrix */
lam = ones(q,1);	/* initialize lambda */

bg = 1; en = 1;
for i (2,q,1);
	L0[i,1:i-1] = a[bg:en]';
bg = en + 1;en = en + i;
endfor;

L0inv = inv(L0);
for i (2,q,1);		/* produce lambda */
	lam[i] = lam[i] - (L0inv[i,1:i-1]^2)*lam[1:i-1];
endfor;

if minc(lam) gt 0;
	Omlik = L0inv*diagrv(eye(q),lam)*L0inv';		/* the correlation matrix */
	pdf = sumc(pdfmvn1mat(z',Xb',invpd(Omlik))) + pdfmvn1(a,a0_,Acap0_);	/* posterior */
else;
	pdf = -100000000000000;	/* assign a large negative number */
endif;

retp(-pdf);		/* negative for proc optmum */
endp;

		/* likelihood for obtaining hessian by BHHH */
		/* called by proc BHHHhess1 */
		/* globals: z, Xb, a0, Acap0_ */

proc likMVPahess(a);
local L0,lam,bg,en,i,L0inv,Omlik,pdf;

L0 = eye(q);		/* lower unitriangular matrix */
lam = ones(q,1);	/* initialize lambda */

bg = 1; en = 1;
for i (2,q,1);
	L0[i,1:i-1] = a[bg:en]';
bg = en + 1;en = en + i;
endfor;

L0inv = inv(L0);
for i (2,q,1);		/* produce lambda */
	lam[i] = lam[i] - (L0inv[i,1:i-1]^2)*lam[1:i-1];
endfor;

if minc(lam) gt 0;
	Omlik = L0inv*diagrv(eye(q),lam)*L0inv';		/* the correlation matrix */
	pdf = pdfmvn1mat(z',Xb',invpd(Omlik));			/* note: no prior here */
else;
	pdf = -100000000000000*ones(n,1);	/* assign a large negative number */
endif;

retp(pdf);
endp;



	/* proc to compute Hessian by BHHH */
	/* declare by using _ophsprc=&BHHHhess1 */

proc BHHHhess1(psi);
local g;

g = gradfd(&likMVPahess,psi);	/* or gradcd */
g = g - meanc(g)';			/* BHHH-2 */

retp(g'g);
endp;


	/* to avoid recalculation of lndet and invpd */
	/* y and m are k*n matrices P is k*k precision */
	/* m can be k*1, if it repeats for every y */
	/* outputs an n*1 vector of log-pdfs */

proc pdfmvn1mat(y,m,P);
local k,n,a,C,e,e2,z;

k = rows(y);
n = cols(y);
a = -(k/2)*ln(2*pi);
C = chol(P);
e = chol(P)*(y-m);
e2 = e^2;
z = a + sumc(ln(diag(C))) - 0.5*sumc(e2);

retp(z);
endp;

		/* proc to generate latent z */
		/* each zij is sampled conditional on zi(-j) */
		/* NOTE: needs to be embedded in a loop for i (1,q,1) */

proc zgen(y,z,Xb,n,Om,ixm,i);
local mu,s2;

{mu,s2} = condmoments(Xb',Om,i|i,z[.,ixm[.,i]]');	/* conditional moments for ith equation */

for ii (1,n,1);
	if y[ii,i] == 0;
		z[ii,i] = rndnna(mu[ii],s2,0);		/* (-infty,0) */
	else;
		z[ii,i] = rndnan(mu[ii],s2,0);		/* [0,infty) */
	endif;
endfor;

retp(z);
endp;



/*=========================/THE MCMC RUN/=========================*/

it = 1;
do while it le capn;
	{beta,bhat,Binv,Xb} = bgen_same_b(z,X,B0b0_,Bcap0_,Om,Ominv);	/* same beta across all equations */
	{a,Om} = OmgenCJ08ARMHoptmum(z,Xb,n,q,a,Om,it-n0);
	Ominv = invpd(Om);
/* COMMENT THE ZGEN PROC BELOW IF TRYING OT ELICIT THE EFFECT OF LATENT DATA SAMPLING */
/* when zgen is commented out, the problem is similar to a continuous data model with y = z */
	for i (1,q,1);
		z = zgen(y,z,Xb,n,Om,ixm,i);
	endfor;
		/* store quantities */
	if it gt n0;
		betam[.,it-n0] = beta;
		bhatm[.,it-n0] = bhat;
		vechBinvm[.,it-n0] = vech(Binv);
		Omm[.,it-n0] = vech(Om);
		cm[.,it-n0] = a;
		Ominvm[.,it-n0] = vech(Ominv);
	endif;
	prnt;	/* printing proc */
it = it + 1;
endo;


		/* PRINTING PROC */
		/* uses global variables */

proc (0) = prnt;
if round(it/prntit)-(it/prntit) == 0;
	?;
	format /rd 4,0;
	?;
	"ITERATION             " it;; " of " capn;
	?;
	format /rd 4,3;
	print "beta " beta';
	print /flush "Om " Om;
	if it gt n0;
		format /rd 4,3;
		print /flush "AR acceptance for Omega so far " (it-n0)/arJ;
		print /flush "MH acceptance for Omega (ARMH) " armhJ/(it-n0);
	endif;
endif;
endp;


