% covariance matrix estimation
% use the decomposition L\SigmaL' = D 
% where D is diagonal
% L lower triangular with ones on diagonal
% simulated example 1 in the paper (all \lambda_{i}'s =1)

clear all;
rand('state', 234567);
randn('state', 18090);

N = 700;
M = 4; % dimension of the covariance matrix
MN = M*N;
nloop = 11000;
burnin = 1000;

% set parameters values
trueSigma = [ 1 .2 .3 -.4; .2 1 .6 .2 ; .3 .6 1 -.2;  -.4 .2 -.2 1 ];

% generate data
u = mvnrnd( zeros(M, 1 ), trueSigma, N );

% prior
nu0 = 1/2 * ( M+1 + (1:M)'- M  ) ;

%initialize
Sig = eye(M);
invSig = inv(Sig);
L = eye(M);;
invL = L;
D = eye(M);
count = 0; % count the no of accepted draws

newnu = nu0 + N;    

rand('state', sum(100*clock)) ;
randn('state', sum(1234*clock) );

disp('Starting MCMC.... ');
disp(' ' );
start_time = clock;

for loop = 1 : nloop
    
    flag = 1; %get a valid candidate draw
    while flag == 1
        Lc = eye( M ); 
        for i = 2 : M
            Dk = inv( eye(i-1) + u(:, 1 : i-1 )' * u(:, 1 : i-1 ) /  D(i,i) ) ;
            dk = - u(:, 1 : i-1 )' * u(:, i ) / D(i,i);
            Lc( i, 1:i-1 ) = mvnrnd( Dk*dk, Dk ); 
        end
        invLc = inv( Lc );
    
        Dc(1,1) = 1;  
        for i = 2 : M
            lambda = diag( Dc );
            ak = invLc( i, 1:i-1 );
            Dc(i,i) = 1 - (ak .^2 ) * lambda( 1 : i-1 ) ;
        end
        
        if ( diag(Dc) > 10^(-4) * ones(M, 1) )
            flag =0;
            Sigc = invLc*Dc*invLc';
            invSigc = inv(Sigc);
        end
    end
    
    lp = 0;
    for i = 2 : M
        Dk = inv( eye(i-1) + u(:, 1 : i-1 )' * u(:, 1 : i-1 ) ) * D(i,i) ;
        dk = - u(:, 1 : i-1 )' * u(:, i ) / D(i,i);
        lp = lp + lmvnpdf( Lc( i, 1:i-1 )', zeros(i-1,1), eye(i-1)) - lmvnpdf( L( i, 1:i-1 )', zeros(i-1,1), eye(i-1)) + lmvnpdf( L( i, 1:i-1 )', Dk*dk, Dk ) - lmvnpdf( Lc( i, 1:i-1 )', Dk*dk, Dk);
    end
    
    w  = reshape(L * u', MN,1);
    wc = reshape(Lc * u',MN,1);
    llike = -N/2*log(prod(diag(D))) - w'*kron(speye(N),inv(sparse(D)))*w/2;
    llikec= -N/2*log(prod(diag(Dc))) -wc'*kron(speye(N),inv(sparse(Dc)))*wc/2;
    urand = rand;

    if (exp(llikec-llike+lp)> urand)
        D = Dc;
        L = Lc;
        invL = invLc;
        Sig = Sigc;
        invSig = invSigc;
        if loop > burnin
            count = count + 1;
        end
    end

    if loop > burnin
        sSigma( loop - burnin, :, : ) = Sig;
    end
    
    if ( mod( loop, 1000 ) ==0 )
        disp(  [ num2str( loop ) ' loops... ' ] )
    end
end
disp( ['MCMC takes '  num2str( etime( clock, start_time) ) ' seconds' ] );
disp(' ' );

Sighat = squeeze( mean( sSigma ) );
Sigstd = squeeze( std( sSigma ) );

