% covariance matrix estimation
% use the decomposition L\SigmaL' = D 
% where D is diagonal
% L lower triangular with ones on diagonal
% simulated example 2 in the paper (sig_3l=sig_32=0; Gibbs)

clear all;
rand('state', 234567);
randn('state', 7771);

N = 700;
M = 4;
MN = M*N;
nloop = 11000;
burnin = 1000;

% set parameters values
trueSigma = [1.2 .9 0 .5; .9 1 0 .3; 0 0 .9 .2; .5 .3 .2 1.1];

% generate data
u = mvnrnd( zeros(M, 1 ), trueSigma, N );

% prior
nu0 = 1/2 * ( M+1 + (1:M)'- M  ) ;
S0 = ones(M,1);

%initialize
Sig = eye(M);
invSig = inv(Sig);
L = eye(M);
D = eye(M);
invL = L;
sSig = zeros( nloop - burnin, M, M );

newnu = nu0 + N;    

rand('state', sum(100*clock)) ;
randn('state', sum(1234*clock) );

disp('Starting MCMC.... ');
disp(' ' );
start_time = clock;

for loop = 1 : nloop
    
    % sample D
    w =  L * u';
    newS = diag( w * w' ) + S0;
    lambda =  1 ./ gamrnd( newnu / 2 , 2 ./ newS );
    D = diag( lambda );
    
    % sample L
    D2 = D(2,2)*inv( 1 + u(:,1)' * u(:,1) );
    d2 = - u(:,1)' * u(:,2) / D(2,2);
    L(2,1) = mvnrnd(D2*d2, D2);
    
    D4 = D(4,4)*inv( eye(3) + u(:,1:3)' * u(:,1:3) );
    d4 = - u(:,1:3)' * u(:,4) / D(4,4);
    L(4,1:3) = mvnrnd(D4*d4, D4);
    
    invL = inv( L );
    Sig = invL * D * invL';

    if loop > burnin
        sSig( loop - burnin, :, : ) = Sig;
    end
    
    if ( mod( loop, 1000 ) ==0 )
        disp(  [ num2str( loop ) ' loops... ' ] )
    end
end
disp( ['MCMC takes '  num2str( etime( clock, start_time) ) ' seconds' ] );
disp(' ' );

Sighat = squeeze( mean( sSig ) );
Sigstd = squeeze( std( sSig ) );