% covariance matrix estimation
% use the decomposition L\SigmaL' = D 
% where D is diagonal
% L lower triangular with ones on diagonal
% simulated example 3 in the paper (\sig11=1; \sig31=\sig42=0)

clear all;
N = 700;
M = 4; % dimension of the covariance matrix
MN = M*N;
nloop = 11000;
burnin = 1000;

% set parameters values
trueSigma =[ 1 0.5 0 0.4; 0.5 0.9 -0.2 0 ; 0 -0.2 1.1 -0.3;  0.4 0 -0.3 0.8 ];

rand('state', 234567);
randn('state', 4761);

% generate data
u = mvnrnd( zeros(M, 1 ), trueSigma, N );

% prior
nu0 = 1/2 * ( M+1 + (1:M)'- M  ) ;
S0 = ones(M,1);

%initialize
Sig = eye(M);
invSig = inv( Sig );
L = eye(M);
D = eye(M);
invL = inv(L);
Lc = L;
count1 = 0;  % count the no. of accepted lambda
count2 = 0;  % count the no. of accepted a
sSigma = zeros(nloop-burnin,M,M);

newnu = nu0 + N;
utemp = reshape( u', M * N, 1 ); % reshape the data

rand('state', sum(100*clock)) ;
randn('state', sum(1234*clock) );


for loop = 1 : nloop
    
    % sample lambda
    % lambda_3 and lambda_4 can use Gibbs while lambda_2 needs a MH step
    w =  L * u';
    newS = diag( w * w' ) + S0;
    lambda =  1 ./ gamrnd( newnu / 2 , 2 ./ newS );
    D(1,1) = 1; % the first diagonal element is restricted to 1
    D(3,3) = lambda(3);
    D(4,4) = lambda(4);
    
    lam2c = lambda(2);
    Dc = D; Dc(2,2) = lam2c;
    Lc = L;
    Lc(4,2) = (Lc(4,3)*Lc(3,2) + 1/lam2c*Lc(2,1)*Lc(4,1)) / (1+1/lam2c*Lc(2,1)^2);
    lp = log(gampdf(1/lam2c, nu0(2)/2 , 2/S0(2))) - log(gampdf(1/D(2,2), nu0(2)/2 , 2/S0(2))) + log(gampdf(1/D(2,2), newnu(2)/2 , 2/newS(2))) - log(gampdf(1/lam2c, newnu(2)/2 , 2/newS(2)));
    
    w  = reshape(L * u', MN,1);
    wc = reshape(Lc * u',MN,1);
    llike = -N/2*log(D(2,2)) - w'*kron(speye(N),inv(sparse(D)))*w/2;
    llikec= -N/2*log(lam2c) - wc'*kron(speye(N),inv(sparse(Dc)))*wc/2;
    urand = rand;

    if (exp(llikec-llike+lp)> urand)
        D(2,2) = lam2c;
        L(4,2) = Lc(4,2);
        invL = inv(Lc);
        Sig = invL*D*invL';
        invSig = inv(Sig);
        if loop > burnin
            count1 = count1 + 1;
        end
    end

    % sample L in one blog
    lp = 0;
    Lc = L;
    
    D2 = D(2,2)*inv( 1 + u(:,1)' * u(:,1) );
    d2 = - u(:,1)' * u(:,2) / D(2,2);
    Lc(2,1) = mvnrnd(D2*d2, D2);
    lp = lp + lmvnpdf( Lc(2,1), 0, D(2,2)) - lmvnpdf( L(2,1), 0, D(2,2)) + lmvnpdf( L(2,1), D2*d2, D2 ) - lmvnpdf( Lc(2,1), D2*d2, D2);
    
    D3 = D(3,3)*inv( 1 + u(:,2)' * u(:,2) );
    d3 = - u(:,2)' *(u(:,3) + L(3,1)*u(:,1))/ D(3,3);
    Lc(3,2) = mvnrnd(D3*d3, D3);
    lp = lp + lmvnpdf( Lc(3,2),0,D(3,3)) - lmvnpdf( L(3,2),0,D(3,3)) + lmvnpdf( L(3,2), D3*d3, D3 ) - lmvnpdf( Lc(3,2), D3*d3, D3) ;
    
    D4 = D(4,4)*inv( eye(2) + u(:, [1 3] )' * u(:, [1 3] ) );
    d4 = - u(:, [1 3])' * (u(:, 4 ) + L(4,2)*u(:,2)) / D(4,4);
    Lc(4,[1 3]) = mvnrnd(D4*d4, D4);
    lp = lp + lmvnpdf( Lc( 4,[1 3])', zeros(2,1), D(4,4)*eye(2)) - lmvnpdf( L( 4,[1 3])', zeros(2,1), D(4,4)*eye(2)) + lmvnpdf( L(4,[1 3])', D4*d4, D4 ) - lmvnpdf( Lc(4,[1 3])', D4*d4, D4);
    
    % restricting \sig31=\sig42=0
    Lc(3,1) = Lc(3,2)*Lc(2,1);
    Lc(4,2) = (Lc(4,3)*Lc(3,2) + 1/D(2,2)*Lc(2,1)*Lc(4,1)) / (1+1/D(2,2)*Lc(2,1)^2);

    invLc = inv( Lc );
    Sigc = invLc * D * invLc';
    invSigc = inv(Sigc);
         
    llike = -N/2*log(det(Sig)) - utemp'*kron(speye(N),invSig)*utemp/2;
    llikec= -N/2*log(det(Sigc)) -utemp'*kron(speye(N),invSigc)*utemp/2;
    urand = rand;

    if (exp(llikec-llike+lp)> urand)
        L = Lc;
        invL = invLc;
        Sig = Sigc;
        invSig = invSigc;
        if loop > burnin
            count2 = count2 + 1;
        end
    end

    if loop > burnin
        sSigma( loop - burnin, :, : ) = Sig;
    end
    
    if ( mod( loop, 1000 ) ==0 )
        disp(  [ num2str( loop ) ' loops... ' ] )
    end
end
Sighat = squeeze( mean( sSigma ) );
Sigstd = squeeze( std( sSigma ) );

